@@ -405,6 +405,7 @@ def main():
         'duk_api_inspect.c',
         'duk_api_memory.c',
         'duk_api_object.c',
+        'duk_api_random.c',
         'duk_api_string.c',
         'duk_api_time.c',
         'duk_api_debug.c',
@@ -510,13 +511,16 @@ def main():
         'duk_unicode.h',
         'duk_unicode_support.c',
         'duk_unicode_tables.c',
+        'duk_util.h',
         'duk_util_bitdecoder.c',
         'duk_util_bitencoder.c',
-        'duk_util.h',
         'duk_util_hashbytes.c',
-        'duk_util_misc.c',
         'duk_util_tinyrandom.c',
         'duk_util_bufwriter.c',
+        'duk_util_double.c',
+        'duk_util_cast.c',
+        'duk_util_memory.c',
+        'duk_util_misc.c',
         'duk_selftest.c',
         'duk_selftest.h',
         'duk_strings.h',
